import numpy as np
from typing import Iterable
from datetime import datetime
import os

try:
    from . import iri_fcore as core
except ImportError:
    raise ImportError("Cannot import compiled IRI library.")


class MissingDataError(Exception):
    pass


def IRI(dt: datetime, alt_range: [float, float, float], lats: Iterable[float], lons: Iterable[float],
        replace_missing: float = np.nan) -> dict:
    try:
        from . import iri_fcore as core
    except ImportError:
        raise ImportError("Cannot import compiled IRI library.")

    lats = np.asarray(lats)
    lons = np.asarray(lons)

    if not len(lats) == len(lons):
        raise ValueError("Lengths of latitude and longitude arrays must be equal.")

    jf = np.ones(50, dtype=bool)
    jf[[3, 4, 5, 11, 21, 22, 25, 27, 28, 29, 32, 33, 34, 35, 36]] = 0
    jmag = 0
    mmdd = 100 * dt.month + dt.day
    dhour = dt.hour + dt.minute / 60 + dt.second / 3600
    oarr = np.zeros(100, dtype=np.float32)
    core.mod.iri_res = np.zeros((20, 1000, len(lats)), dtype=np.float32)
    datadir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/')
    core.mod.iri_core(jf, jmag, lats, lons, dt.year, mmdd, dhour + 25., alt_range[0], alt_range[1], alt_range[2], oarr,
                      datadir)
    ne = core.mod.iri_res[0].transpose()
    te = core.mod.iri_res[3].transpose()
    nalts = int((alt_range[1] - alt_range[0]) / alt_range[2]) + 1
    ne = ne.reshape((len(lats), -1))[:, :nalts]
    te = te.reshape((len(lats), -1))[:, :nalts]
    res = {
        'ne': np.where(ne < 0, replace_missing, ne),
        'te': np.where(te < 0, replace_missing, te),
    }
    return res



from numpy.distutils.core import Extension
from numpy.distutils.core import setup
from setuptools import find_packages
import os

fsources = ['iri_fcore.pyf', 'iri_fcore.f90', 'cira.for', 'igrf.for', 'iridreg.for', 'iriflip.for',
            'irifun.for', 'irisub.for', 'iritec.for']

here = os.path.dirname(os.path.realpath(__file__))

iri_fcore = Extension(
    'iricore.iri_fcore',
    sources=[os.path.join(here, 'src/iricore/src/') + fs for fs in fsources],
    extra_f90_compile_args=['-fPIC', '-O3', '-w', '-Wtabs'],
    extra_f77_compile_args=['-fPIC', '-O3', '-w', '-Wtabs'],
)

if __name__ == "__main__":
    # noinspection PyInterpreter
    setup(name='iricore',
          version="1.0.6",
          author="lap1dem",
          author_email="vadym.bidula@gmail.com",
          url='https://github.com/lap1dem/iricore',
          download_url='https://github.com/lap1dem/iricore/archive/refs/tags/1.0.0.tar.gz',
          license='LICENSE',
          description='A python interface to iri2016 Fortran code',
          long_description=open('README.md').read(),
          long_description_content_type="text/markdown",
          install_requires=[
              "numpy>=1.21",
          ],
          package_dir={"": "src"},
          packages=find_packages(where="src"),
          ext_modules=[iri_fcore],
          include_package_data=True,
          package_data={
              "iricore": [
              "data/ccir/*.asc",
              "data/igrf/*.dat",
              "data/index/*.dat",
              "data/mcsat/*.dat",
              "data/ursi/*.asc",
              ],
          },
          python_requires=">=3.7",
          classifiers=[
              "Development Status :: 5 - Production/Stable",
              "License :: OSI Approved :: MIT License",
              "Programming Language :: Python :: 3",
              "Programming Language :: Python :: 3.7",
              "Programming Language :: Python :: 3.8",
              "Programming Language :: Python :: 3.9",
              "Programming Language :: Python :: 3.10",
          ],
          )



def update():
    from urllib import request, error
    import os
    location = os.path.dirname(os.path.abspath(__file__))
    try:
        request.urlretrieve(
            'https://chain-new.chain-project.net/echaim_downloads/apf107.dat',
            os.path.join(location, 'data/index/apf107.dat')
        )
        request.urlretrieve(
            'https://chain-new.chain-project.net/echaim_downloads/ig_rz.dat',
            os.path.join(location, 'data/index/ig_rz.dat')
        )
        print('The index data was successfully updated!')
    except error.URLError:
        print('Something went wrong. Check internet connection.')
